# Define _make_verbose if it doesn't exist (RHEL8)
%{!?_make_verbose:%define _make_verbose V=1 VERBOSE=1}
%define njs_version 0.8.9
%define quickjs_version 2024.07.24

Name:           nginx-mod-njs
Version:        0.8.9
Release:        1%{?dist}
Summary:        Nginx njs module
License:        2-clause BSD-like license
URL:            https://github.com/nginx/njs
Packager:       Teddysun <i@teddysun.com>
Source0:        https://github.com/nginx/njs/archive/refs/tags/%{version}.tar.gz#/njs-%{version}.tar.gz
Source1:        quickjs-%{quickjs_version}.tar.xz
Patch0:         quickjs_quickjs.makefile.patch

BuildRequires:  gcc
BuildRequires:  nginx-mod-devel
BuildRequires:  zlib-devel
BuildRequires:  pcre2-devel
BuildRequires:  libedit-devel
BuildRequires:  libxml2-devel
BuildRequires:  libxslt-devel
BuildRequires:  libatomic
Requires:       nginx
Requires:       nginx-mod-stream

%define WITH_CC_OPT $(echo %{optflags} $(pcre2-config --cflags))
%define WITH_LD_OPT -Wl,-z,relro -Wl,-z,now -Wl,-as-needed
%define bdir %{_builddir}/%{name}-%{version}

%description
NGINX JavaScript, also known as NJS, is a dynamic module for NGINX 
that enables the extension of built-in functionality using familiar JavaScript syntax.
The NJS language is a subset of JavaScript, compliant with ES5 (ECMAScript 5.1 Strict Variant) 
with some ES6 (ECMAScript 6) and newer extensions. See compatibility for more details.

%prep
%setup -qcTn %{name}-%{version}

tar xvzfo %{SOURCE0}
ln -s njs-* njs
tar xvJfo %{SOURCE1}
ln -s quickjs-* quickjs

cd %{bdir}/quickjs
%patch0 -p1

%build
_nproc=`getconf _NPROCESSORS_ONLN`
if [ $_nproc -gt 1 ]; then
    _make_opts="-j$_nproc"
fi
cd %{bdir}/quickjs-%{quickjs_version} && CFLAGS="%{WITH_CC_OPT} -fPIC" make $_make_opts libquickjs.a
%define __with_quickjs --with-quickjs
cd %{bdir}/njs-%{njs_version} && sed -i.bak 's,-lquickjs.lto ,-lquickjs.lto -latomic ,g' auto/quickjs && sed -i.bak 's,-lquickjs ,-lquickjs -latomic ,g' auto/quickjs && ./configure %{__with_quickjs} --cc-opt="%{WITH_CC_OPT} -I%{bdir}/quickjs-%{quickjs_version}/" --ld-opt="%{WITH_LD_OPT} -L%{bdir}/quickjs-%{quickjs_version}/" && make $_make_opts njs && mv build build-cli
cd %{bdir}

%undefine _strict_symbol_defs_build
cp -a "%{_nginx_srcdir}" "%{_nginx_buildsrcdir}"
cd "%{_nginx_buildsrcdir}"
./configure --with-compat --with-stream --add-dynamic-module=$(realpath %{_nginx_modsrcdir}/njs-%{njs_version}/nginx) --builddir=$(realpath %{_nginx_modbuilddir}) \
            --with-cc-opt="%{WITH_CC_OPT} -I%{bdir}/quickjs-%{quickjs_version}/" \
            --with-ld-opt="%{WITH_LD_OPT} -L%{bdir}/quickjs-%{quickjs_version}/"
make %{?_smp_mflags} modules


%install
pushd %{_vpath_builddir}
%{__rm} -rf %{buildroot}
%{__mkdir} -p %{buildroot}%{_bindir}
%{__install} -m755 %{bdir}/njs-%{njs_version}/build-cli/njs %{buildroot}%{_bindir}/

%{__mkdir} -p %{buildroot}%{nginx_moddir}
for so in `find %{bdir}/ -type f -name "*.so"`; do
%{__install} -m755 $so %{buildroot}%{nginx_moddir}
done
popd

%{__install} -dm 0755 %{buildroot}%{nginx_modconfdir}
echo 'load_module "%{nginx_moddir}/ngx_http_js_module.so";'    > %{buildroot}%{nginx_modconfdir}/mod-njs.conf
echo 'load_module "%{nginx_moddir}/ngx_stream_js_module.so";'  >> %{buildroot}%{nginx_modconfdir}/mod-njs.conf

%files
%license njs/LICENSE
%doc njs/CONTRIBUTING.md njs/README.md njs/CODE_OF_CONDUCT.md njs/SECURITY.md njs/SUPPORT.md
%{_bindir}/njs
%{nginx_moddir}/*.so
%{nginx_modconfdir}/mod-njs.conf

%changelog
* Sun Mar 16 2025 Teddysun <i@teddysun.com> - 0.8.9-1
- Initial packaging version 0.8.9 for nginx 1.27.4
